#ifndef __VTKEXPORT_H
#define __VTKEXPORT_H

#include <fstream>
#include <vector>

/**
 * Exportiert ein Vierecksgitter ins vtkUnstructuredGrid-Format, das
 * beispielsweise mit Paraview geoeffnet und visualisiert werden kann.
 *
 * Als Eingabedaten werden ein Vektor aller Gitterpunkte (point) ohne
 * Duplikate und ein Vektor von Punkindizes fuer jedes Element benoetigt.
 * Die Elemente sind hierbei wie folgt indiziert:
 *
 *          3 ---------- 2
 *          |            |
 *          |            |
 *          |            |
 *          |            |
 *          |            |
 *          0------------1
 *
 * Ein Beispiel finden Sie in der Datei "vtkExportDemo.cpp".
 *
 * Changelog:
 * 21.06.2016: Typ von elements geaendert, damit der Vektor der Elemente nicht mehr
 *             aus Gleitkommazahlen bestehen muss.
 **/
template<typename VecVecRType, typename VecVecIType>
void exportGrid(VecVecRType &points, VecVecIType &elements, const char* filename) {
  std::ofstream out(filename);
  // Schreibe Header:
  out << "<VTKFile type=\"UnstructuredGrid\">\n";
  out << "<UnstructuredGrid>\n";
  out << "<Piece NumberOfPoints=\"" << points.size() << "\" NumberOfCells=\"" << elements.size() << "\">\n";

  // Schreibe die Punktkoordinaten:
  out << "<Points>\n";
  out << "<DataArray type=\"Float32\" NumberOfComponents=\"3\">\n";
  for(unsigned int i = 0; i < points.size(); ++i) {
    for(unsigned int c = 0; c < 3; ++c) {
      out << points[i][c] << ' ';
    }
    out << '\n';
  }

  out << "</DataArray>\n";
  out << "</Points>\n";

  // Schreibe die Zelldaten (Eckpunkte der Vierecke):
  out << "<Cells>\n";
  out << "<DataArray type=\"Int32\" Name=\"connectivity\" format=\"ascii\">\n";
  for(unsigned int i = 0; i < elements.size(); ++i) {
    for(unsigned int j = 0; j < 4; ++j) {
      out << elements[i][j] << ' ';
    }
    out << '\n';
  }
  out << "</DataArray>\n";

  // Die Offsets geben an, an welcher Stelle des Cells-Array das i + 1. Element
  // beginnt.
  out << "<DataArray type=\"Int32\" Name=\"offsets\" format=\"ascii\">\n";
  for(unsigned int i = 0; i < elements.size(); ++i) {
    out << (i + 1)*4 << '\n';
  }
  out << "</DataArray>\n";

  // Hier werden die Typen der Elemente aufgelistet. Da nur Quads verwendet
  // werden ist der Typ immer 9.
  out << "<DataArray type=\"UInt8\" Name=\"types\" format=\"ascii\">\n";
  // Quads sind Typ 9
  for(unsigned int i = 0; i < elements.size(); ++i) {
    out << 9 << '\n';
  }
  out << "</DataArray>\n";
  out << "</Cells>\n";
  out << "</Piece>\n";
  out << "</UnstructuredGrid>\n";
  out << "</VTKFile>\n";

  out.close();
}

#endif /* __VTKEXPORT_H */
